//  loop.hpp: loop view

//  Copyright Takeshi Mouri 2006.
//  Use, modification, and distribution are subject to the
//  Boost Software License, Version 1.0. (See accompanying file
//  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef HAMIGAKI_IOSTREAMS_LOOP_HPP
#define HAMIGAKI_IOSTREAMS_LOOP_HPP

#include <hamigaki/iostreams/concatenate.hpp>
#include <boost/iostreams/categories.hpp>
#include <boost/iostreams/detail/ios.hpp>
#include <boost/iostreams/read.hpp>
#include <boost/iostreams/seek.hpp>
#include <boost/iostreams/traits.hpp>

namespace hamigaki { namespace iostreams {

template<class Source>
class loop_source
{
public:
    typedef typename boost::iostreams::
        char_type_of<Source>::type char_type;

    struct category :
        boost::iostreams::input,
        boost::iostreams::device_tag,
        boost::iostreams::closable_tag {};

    loop_source(const Source& src, int count=-1)
        : src_(src), count_(count)
    {
    }

    std::streamsize read(char_type* s, std::streamsize n)
    {
        if (count_ == 0)
            return -1;

        std::streamsize total = 0;
        while (n != 0)
        {
            std::streamsize amt = boost::iostreams::read(src_, s, n);
            if (amt == -1)
            {
                if (count_ != -1)
                {
                    if (--count_ == 0)
                        break;
                }
                boost::iostreams::seek(src_, 0, BOOST_IOS::beg);
            }
            else
            {
                total += amt;
                s += amt;
                n -= amt;
            }
        }
        return (total != 0) ? total : -1;
    }

    void close()
    {
        boost::iostreams::close(src_, BOOST_IOS::in);
    }

private:
    Source src_;
    int count_;
};

template<class Source>
inline loop_source<Source>
loop(const Source& src, int count=-1)
{
    return loop_source<Source>(src, count);
}

} } // End namespaces iostreams, hamigaki.

HAMIGAKI_IOSTREAMS_CATABLE(hamigaki::iostreams::loop_source, 1)

#endif // HAMIGAKI_IOSTREAMS_LOOP_HPP
